/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xmpl.h"

FD_xmpl *fdui;


int set_alarm(int mode)
{
char *ptr;
int a;
int error;
char temp[80];
char temp1[80];

if(debug_flag)
	{
	printf("set_alarm(): arg mode=%d\n", mode);
	}

switch(mode)
	{
	case SET_ALARM_TIME:
		sprintf(temp1,\
		"Alarm time (format hh:mm, it is now %02d:%02d)?",\
		hour, minute);

		sprintf(temp,\
		"%02d:%02d",\
		alarm_hour, alarm_minute);

		ptr = (char *) fl_show_input(temp1, temp);
		if(! ptr) return 0;
		a = sscanf(ptr, "%2d:%2d", &alarm_hour, &alarm_minute);

		error = 0;
		if(a != 2) error = 1;
		if(alarm_hour < 0) error = 1;
		if(alarm_hour > 23) error = 1;
		if(alarm_minute < 0) error = 1;
		if(alarm_minute > 59) error = 1;
		if(error)
			{
			fl_show_alert(\
			"Incorrect time format", "ALARM NOT SET", "", 0);

			return 0;
			}
	
		return 1;
		break;
	case SLEEP:
		if(power_down_flag && !sleep_mode)
			{
			/* mouse on NO */
			if(! fl_show_question(
"This will power down your PC if the sleep timer expires\nARE YOU VERY SURE?", 0) )
				{
				return;
				}
			}

		sleep_mode = 1 - sleep_mode;
		set_alarm_menu();
		return 1;
		break;
	case ALARM:
		alarm_mode = 1 - alarm_mode;
		set_alarm_menu();
		return 1;
		break;
	case POWER_DOWN_PC_ON_SLEEP:
		if( !power_down_flag && sleep_mode)
			{
			/* mouse on NO */
			if(! fl_show_question(
"This will power down your PC if the sleep timer expires\nARE YOU VERY SURE?", 0) )
				{
				return;
				}
			} /* end if ! power down flag */

		power_down_flag = 1 - power_down_flag;

		set_power_down_menu(power_down_flag);

		save_general_settings();
		return;
		break;
	default:
		return 0;
		break;
	} /* end switch mode */

return 1;
} /* end function set_alarm */


int set_alarm_menu()
{
if(debug_flag)
	{
	printf(\
	"set_alarm_menu(): arg none using alarm_mode=%d sleep_mode=%d\n",\
	alarm_mode, sleep_mode);
	}

if(sleep_mode)
	{
	fl_set_menu_item_mode(fdui -> main_form_alarm_menu,\
	SLEEP, FL_PUP_CHECK);
	fl_set_object_lcolor(fdui -> main_form_volume_slider, FL_DEEPPINK);
	}
else
	{
	fl_set_menu_item_mode(fdui -> main_form_alarm_menu,\
	SLEEP, FL_PUP_BOX);
	fl_set_object_lcolor(fdui -> main_form_volume_slider, FL_BLACK);
	}

if(alarm_mode)
	{
	fl_set_menu_item_mode(fdui -> main_form_alarm_menu,\
	ALARM, FL_PUP_CHECK);
	}
else
	{
	fl_set_menu_item_mode(fdui -> main_form_alarm_menu,\
	ALARM, FL_PUP_BOX);
	} /* end alarm of */

return 1;
} /* end function set_alarm_menu */


int test_alarm()
{
char temp[80];
static int old_minute;

/* get present date and time */
ogettime(weekday, month, &monthday, &hour, &minute, &second, &year);

if(sleep_mode)
	{
	if( (minute != old_minute) && (selected_volume > 0) )
		{
		selected_volume--;
		set_mixer_volume(selected_volume);
		old_minute = minute;

		if(selected_volume == 0)
			{
			play_stop();

			if(power_down_flag)
				{
//				power_down_flag = 0;
//				save_general_settings();

				power_down_pc();
				}
			} /* end if end sleep in period */

		} /* end decrease volume */

	} /* end if sleep */
if(alarm_mode)
	{
	if( (hour == alarm_hour) && (minute == alarm_minute) && (second == 0) )
		{
		fl_set_object_color(fdui -> main_form_stop_button, FL_RED, FL_RED);

		sprintf(temp, "%02d:%02d", hour, minute);
		fl_set_object_label(fdui -> main_form_stop_button, temp);
		
		time_match_flag = 1;

		selected_volume = ALARM_VOLUME;
		set_mixer_volume(selected_volume);
		play_start();

		alarm_mode = 0;
		set_alarm_menu();

		return 1;
		} /* end if time match */
	else /* alarm off */
		{
		fl_set_object_color(fdui -> main_form_stop_button,\
			FL_DARKORANGE, FL_RED);

		sprintf(temp, "STOP\n%02d:%02d / %02d:%02d:%02d",\
		alarm_hour, alarm_minute, hour, minute, second);
		fl_set_object_label(fdui -> main_form_stop_button, temp);
		} /* end if alarm set and waiting for alarm */

	} /* end if alarm on or sleep */
else
	{
	fl_set_object_color(fdui -> main_form_stop_button, FL_COL1, FL_RED);

	sprintf(temp, "STOP\n%02d:%02d:%02d", hour, minute, second);
	fl_set_object_label(fdui -> main_form_stop_button, temp);

	return 1;
	} /* end if alarm off */

return 1;
} /* end if function test_alarm */


int power_down_pc()
{
FILE *pptr;
char temp[80];

if(debug_flag)
	{
	printf("power_down_pc(): arg none\n");
	}

sprintf(temp, "%s", POWER_OFF_COMMAND);
pptr = popen(temp, "w");
if(pptr) pclose(pptr);

return 1;
} /* end function power_down_pc */

