/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xmpl.h"

extern FD_xmpl *fdui;

int last_found_line;


struct play_list *lookup_play_list(char *name)
{
struct play_list *pa;

for(pa = play_listtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}

/* not found */
return 0;
}/* end function lookup_play_list */


struct play_list *install_play_list_at_end_of_list(char *name)
{
struct play_list *plast, *pnew;

if(debug_flag)
	{
	fprintf(stdout,\
	"install_play_list_at_end_off_list(): arg name=%s\n", name);
	}

pnew = lookup_play_list(name);
if(pnew) return(pnew);/* already there */

/* create new structure */
pnew = (struct play_list *) calloc(1, sizeof(*pnew) );
if(! pnew) return 0;
pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* get previous structure */
plast = play_listtab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previuos structure pointers */
if( !play_listtab[0] ) play_listtab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
play_listtab[1] = pnew;

return pnew;
}/* end function install_play_list */


int delete_play_list(int line)/* delete entry from double linked list */
{
int a;
struct play_list *pa, *pprev, *pdel, *pnext;
char name[80];
char temp[4096];

if(debug_flag)
	{
	fprintf(stdout, "delete_play_list(): arg line=%d\n", line);
	}

pa = play_listtab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return 0;

	if(line != pa -> browser_line)
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* delete file */
	sprintf(temp, "%s", pa -> name);
	a = unlink(temp);
	if(a < 0)
		{
		fl_show_alert("Could not delete file", temp, "", 0);
		}

	/* delete backup file */
	sprintf(temp, "%s~", pa -> name);
	a = unlink(temp);
	if(a < 0)
		{
		fl_show_alert("Could not delete backup file", temp, "", 0);
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify play_listtab[0] */
	if(pprev == 0) play_listtab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify play_listtab[1] */
	if(pnext == 0) play_listtab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	free(pdel -> name);
	free(pdel); /* free structure */

	/* return OK deleted */
	return 1;
	}/* end for all structures */
}/* end function delete_play_list */


int delete_all_play_lists()/* delete all entries from table */
{
struct play_list *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_play_lists() arg none\n");
	}

while(1)
	{	
	pa = play_listtab[0];
	if(! pa) break;
	play_listtab[0] = pa -> nxtentr;

	free(pa -> name);
	free(pa);/* free structure */
	}/* end while all structures */

play_listtab[1] = 0;
return 1;
}/* end function delete_all_play_lists */


int load_play_lists()
{
int a;
FILE *fileptr;
char temp[READSIZE];
char pathfilename[TEMP_SIZE];
struct play_list *pa;
char name[TEMP_SIZE];
int browser_line; /* used to lookup name (= id) if clicked in display,
						not saved on disk, assigned each time list
						is displayed */
int status;
int line;
int spare;
int selected_entry;

if(debug_flag)
	{
	fprintf(stdout, "load_play_lists() arg none\n");
	}
	
delete_all_play_lists();

sprintf(pathfilename, "%s/.xmpl/%s",\
home_dir, database_file_name);
fileptr = fopen(pathfilename, "r");
if(! fileptr)
	{
	fl_show_alert("could not open file", pathfilename, "for read", 1);

	if(debug_flag)
		{
		fprintf(stdout, "could not load file %s\n", pathfilename);
		}

	return 0;
	} 

/* for all lines in database_file_name */
line = 0;
while(1)
	{
	a = readline(fileptr, temp);
	if(a == EOF) return 1;/* empty file or last line */

	strcpy(name, temp);

	/* increment line counter for error reporting */
	line ++;

	a = readline(fileptr, temp);
	if(a == EOF) return 1;/* empty file or last line */
	
	sscanf(temp,\
	"%d %d %d",\
	&browser_line,
	&status,\
	&selected_entry);

	/* increment line counter for error reporting */
	line++;

	/* create a structure entry */
	pa = install_play_list_at_end_of_list(name);
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"load_play_lists: cannot install play_list %s\n", temp);
			}
		return 0;
		}

	pa -> status = status;
	pa -> browser_line = browser_line;
	pa -> selected_entry = selected_entry;

	}/* end while all lines in database_file_name */

return 1;
}/* end function load_play_lists */


int save_play_lists()
{
FILE *fileptr;
char pathfilename[TEMP_SIZE];
char pathfilename2[TEMP_SIZE];
struct play_list *pa;

if(debug_flag)
	{
	fprintf(stdout, "save_play_lists() arg none\n\
	using database_file_name=%s\n",\
	database_file_name);
	}

sprintf(pathfilename, "%s/.xmpl/%s.tmp",\
home_dir, database_file_name);
fileptr = fopen(pathfilename, "w");
if(! fileptr)
	{
	fl_show_alert("could not open file", pathfilename, "for write", 1);

	if(debug_flag)
		{
		fprintf(stdout,\
		"could not open file %s for write\n", pathfilename);
		}
	return 0;
	} 

for(pa = play_listtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(! pa -> name) continue;

	fprintf(fileptr, "%s\n", pa -> name);

	fprintf(fileptr, "%d %d %d\n",\
	pa -> browser_line,\
	pa -> status,\
	pa -> selected_entry);

	}/* end while all elements in list */

fclose(fileptr);

if(debug_flag)
	{
	fprintf(stdout, "save_play_lists(): backing up %s\n", database_file_name);
	}

/* set some path file names */
sprintf(pathfilename, "%s/.xmpl/%s",\
home_dir, database_file_name);

sprintf(pathfilename2, "%s/.xmpl/%s~",\
home_dir, database_file_name);

/* unlink the old .dat~ */
unlink(pathfilename2);

/* rename .dat to .dat~ */
if( rename(pathfilename, pathfilename2) == -1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "save_play_lists(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	}

/* rename .tmp to .dat */
sprintf(pathfilename2, "%s/.xmpl/%s.tmp",\
home_dir, database_file_name);
if( rename(pathfilename2, pathfilename) == -1)
	{
	fl_show_alert("could not rename", pathfilename2, pathfilename, 1);

	if(debug_flag)
		{
		fprintf(stdout, "save_play_lists(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	return 0;
	}

/* ok */
return 1;
}/* end function save_play_lists */


int show_play_lists(int position)
{
char temp[TEMP_SIZE];
struct play_list *pa;
char formatstr[20];
int browser_line;
int browser_topline;
int first_new_play_list_line;
int find_first_new_play_list_flag;
int maxline;
int a;
int next_start_play_list;
struct play_list *pnext;
int hour, minute, second, play_list;

if(debug_flag)
	{
	fprintf(stdout, "show_play_lists(): arg position=%d\n", position);
	}

/* remember vertical slider position */
browser_topline =\
	fl_get_browser_topline(fdui -> main_form_play_list_browser);

fl_freeze_form(fdui -> main_form);

fl_clear_browser(fdui -> main_form_play_list_browser);

first_new_play_list_line = -1;/* keep gcc -Wall from complaining */
find_first_new_play_list_flag = 1;
browser_line = 1;
for(pa = play_listtab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	strcpy(formatstr, "@f");
	/* black is default color */
	strcat(formatstr, "@C0"); /* black */
//	if(!pa -> status) strcat(formatstr, "@C0"); /* black */
	if(pa -> status & NO_PLAY) strcat(formatstr, "@C18"); /* dark green */
//	if(pa -> status & TOO_LONG) strcat(formatstr, "@C1"); /* red */ 
//	if(pa -> status & TXT_HOLD) strcat(formatstr, "@C24"); /* blue */
//	if(pa -> status & XXXXXXX) strcat(formatstr, "@C5"); /* magenta */
	
//	if(highlight_play_lists(pa) ) strcat(formatstr, "@_");/* underscore */

	sprintf(temp, "%s %s",\
		formatstr, pa -> name);
	
	fl_add_browser_line(fdui -> main_form_play_list_browser, temp);
	pa -> browser_line = browser_line; /* first one is 1 */

	browser_line++;
	}/* end for all elements in list */

/* re adjust browser for same position of top line (vertical slider) */	
if(position == SHOW_UNREAD_AT_TOP)
	{
	fl_set_browser_topline(fdui -> main_form_play_list_browser,\
	first_new_play_list_line);
	}
if(position == SHOW_BOTTOM)
	{
	/* make the status of the last play_list visible */
	maxline = fl_get_browser_maxline(fdui -> main_form_play_list_browser);
	fl_set_browser_topline(fdui -> main_form_play_list_browser, maxline);
	}
if(position == SHOW_TOP)
	{
	fl_set_browser_topline(fdui -> main_form_play_list_browser, 1);
	}
if(position == SHOW_SAME_POSITION)
	{
	fl_set_browser_topline(fdui -> main_form_play_list_browser,\
	browser_topline);
	}	
fl_unfreeze_form(fdui -> main_form);

return 1;
}/* end function show_play_lists */


int play_list_to_line(char *text, int *line)
{
struct play_list *pa;

if(debug_flag)
	{
	fprintf(stdout, "play_list_to_line(): arg text=%s\n", text);
	}

pa = lookup_play_list(text);
if(! pa) return 0;

*line = pa -> browser_line;
return 1;
}/* end function play_list_to_line */


int search_and_select_next_play_list()
{
int a;
struct play_list *pa;
char *keyword;
int case_sensitive_flag;
int found;

if(debug_flag)
	{
	fprintf(stdout, "search_and_select_next_play_list(): arg none\n");
	}

keyword = (char *)fl_get_input(fdui -> main_form_search_input_field);
if(! keyword) return 0;

case_sensitive_flag =\
fl_get_button(fdui -> main_form_search_case_button);

found = 0;
/* pa points to next entry */
for(pa = play_listtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* search from last position (set to zero if keyword changes */
	if(pa -> browser_line <= last_found_line) continue;
	
	if(search_in_for(pa -> name, keyword, case_sensitive_flag) )
		{
		found = 1;
		}

	if(found)
		{
		/*
		maintain visible indication that line is selected with single
		click
		*/
		fl_select_browser_line(fdui -> main_form_play_list_browser,\
		pa -> browser_line);

		/* show the entry */
		fl_set_browser_topline(fdui -> main_form_play_list_browser,\
		pa -> browser_line);

		/* remember where we are */
		last_found_line = pa -> browser_line;
		
		selected_title = atoi(pa -> name);

		/* return found */
		return 1;
		}/* end if found */
	}/* end for all play_lists */

if(last_found_line) fl_show_alert(keyword, "end of list", "", 0);
else fl_show_alert(keyword, "not found", "", 0);

last_found_line = 0;

/* return not found */
return 0;
}/* end function search_and_select_play_list */


int set_play_list_status(char *play_list, int status)
{
struct play_list *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"set_play_list_status(): arg play_list=%s status=%d\n", play_list, status);
	}

/* argument check */
if(! play_list) return 0;

pa = lookup_play_list(play_list);
if(! pa) return 0;

pa -> status = status;

return 1;
}/* end function set_play_list_status */
 

int get_play_list_status(char *play_list, int *status)
{
struct play_list *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"get_play_list_status(): arg play_list=%s\n", play_list);
	}

/* argument check */
if(! play_list) return 0;

pa = lookup_play_list(play_list);
if(! pa) return 0;

*status = pa -> status;

return 1;
}/* end function get_play_list_status */
 

struct play_list *install_play_list_in_position(char *name)
{
int i;
struct play_list *pa, *pprev, *plast, *plastmatch, *pnew, *pnext;
struct play_list *lookup_play_list();
char *test_string;
char *ptr1, *ptr2;

if(debug_flag)
	{
	fprintf(stdout,\
	"install_play_list_in_position():\nname=%s\n",\
	name);
	}
	
/* argument check */
if(! name) return 0;

plastmatch = 0;
for(pa = play_listtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* test if playlist with this name already exists */
	if(strcmp(name, pa -> name) == 0)
		{
		fl_show_alert(\
		"A play list with name", name, "already exists, command cancelled", 0);

		return 0;
		}

	/* test if to far */
	if(strcmp(name, pa -> name) < 0) break;

	/* remember this position */
	plastmatch = pa;
	} /* end for all entries in list */

if(debug_flag)
	{
	fprintf(stdout,\
	"ouside for plastmatch=%lu\n", plastmatch);
	}

/* plastmatch now points to insert point, or zero if not found, */

/* create new structure */
pnew = (struct play_list *) calloc(1, sizeof(*pnew) );
if(! pnew) return 0;
pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* add to end list if no match */
if(! plastmatch)
	{
	/* get previous structure */
	plast = play_listtab[1]; /* end list */

	/* set new structure pointers */
	pnew -> nxtentr = 0; /* new points top zero (is end) */
	pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

	/* set previous structure pointers */
	if( !play_listtab[0] ) play_listtab[0] = pnew; /* first element in list */
	else plast -> nxtentr = pnew;

	/* set array end pointer */
	play_listtab[1] = pnew;

	/* return pointer to new structure */
	return(pnew);
	}/* end if ! plastmatch */

/* plastmatch now holds insert position, insert play_list */
/* insert the structure AFTER plastmatch */
		
/* get previous and next structure */
pnext = plastmatch -> nxtentr;
pprev = plastmatch;

/* set pointers for new struture */
pnew -> nxtentr = pnext;
pnew -> prventr = pprev;

/* set pointers for next structure */
if(pnext == 0) play_listtab[1] = pnew;
else pnext -> prventr = pnew;		

/* set pointer for previous structure */
if(pprev == 0) play_listtab[0] = pnew;
else pprev -> nxtentr = pnew;
		
/* mark it as a new entry */
pnew -> status = NEW_ENTRY;

/* return pointer to new structure */
return pnew;
}/* end function install_play_list_in_position */


int add_play_list_in_position(char *text)
{
char temp[512];
struct play_list *pa;
struct play_list *pnext;
int next_start_play_list;

if(debug_flag)
	{
	printf("add_play_list_in_position(): arg text=%s\n", text);
	}

/* sets status to new entry if found not to exist */
pa = install_play_list_in_position(text);
if(! pa) return 0;

/* 
sort the list, cause if item < first entry was entered it
would be at the end
*/
sort_play_lists();

return 1;
}/* end function add_play_list_in_position */


int sort_play_lists()
/*
sorts the double linked list with criterium set by  mode.
doing some sort of bubble sort.
*/
{
struct play_list *pa;
struct play_list *pb;
int swap_flag;

if(debug_flag)
	{
	printf("sort_play_lists(): arg none\n");
	}

while(1)/* go through list again and again */
	{
	swap_flag = 0;
	for(pa = play_listtab[0]; pa != 0; pa = pa -> nxtentr)
		{
		pb = pa -> prventr;
		if(pb)
			{
			if(strcmp(pa -> name, pb -> name) < 0)
				{
				/* indicate position was swapped */
				swap_flag = swap_play_list_position(pa , pb);
				}/* end if condition */
			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, list is sorted */
	if(! swap_flag) break;
	}/* end while go through list again and again */

return 1;
}/* end function sort_play_lists */


int swap_play_list_position(struct play_list *ptop, struct play_list *pbottom)
{
struct play_list *punder;
struct play_list *pabove;

if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): swapping top=%lu bottom=%lu\n", ptop, pbottom);
	}

/* get one below the bottom */
punder = pbottom -> prventr;/* could be zero if first entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): punder=%lu\n", punder);
	}

/* get the one above the top */
pabove = ptop -> nxtentr;/* could be zero if last entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): pabove=%lu\n", pabove);
	}

/* the next pointer in punder (or play_listtab[0]) must now point to ptop */
if(! punder)
	{
	play_listtab[0] = ptop; 
	}
else
	{
	punder -> nxtentr = ptop;
	}

/* the prev pointer in in ptop must now point to punder */
ptop -> prventr = punder;/* could be zero if first entry */

/* the next pointer in ptop must now point to pbottom */
ptop -> nxtentr = pbottom;

/* the next pointer in pbottom must now point to pabove */
pbottom -> nxtentr = pabove;

/* mark last one in play_listtabtab */
if(! pabove)
	{
	play_listtab[1] = pbottom;
	}
else
	{
	/* the prev pointer in pabove must now point to pbottom */
	pabove -> prventr = pbottom;
	}

/* the prev pointer in pbottom must now point to ptop */
pbottom -> prventr = ptop;

/* return swapped */
return 1;
}/* end function swap_play_list_position */


int swap_play_lists(int line)
/*
sorts the double linked list with criterium set by  mode.
doing some sort of bubble sort.
*/
{
struct play_list *pa;
struct play_list *pb;
int swap_flag;

for(pa = play_listtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	pb = pa -> prventr;
	if(pb)
		{
		if(pa -> browser_line == line)
			{
			/* indicate position was swapped */
			swap_flag = swap_play_list_position(pa , pb);

			return 1;
			}/* end if condition */
		}/* end if pb */
	}/* end for all entries */

/* not found */
return 0;
}/* end funcion swap_play_lists */


char *line_to_play_list_name(int line)
{
struct play_list *pa;

if(debug_flag)
	{
	printf("line_to_name(): arg line=%d\n", line);
	}

for(pa = play_listtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line) return pa -> name;

	}/* end for all entries */

/* not found */
return 0;
} /* end function line_to_play_list_name */


int set_selected_entry_in_play_list(int line)
{
struct play_list *pa;

if(debug_flag)
	{
	printf("set_selected_entry_in_play_list(): arg line=%d\n", line);
	}

for(pa = play_listtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == selected_play_list)
		{
		pa -> selected_entry = line;
		return 1;
		}

	}/* end for all entries */

/* not found */
return 0;
} /* end function set_selected_entry_in_play_list */


int get_selected_entry(int line, int *selected)
{
struct play_list *pa;

if(debug_flag)
	{
	printf("get_selected_entry(): arg line=%d\n", line);
	}

for(pa = play_listtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		*selected = pa -> selected_entry;
		return 1;
		}

	}/* end for all entries */

/* not found */
return 0;
} /* end function get_selected_entry */


int rename_play_list(int line)
{
int a;
struct play_list *pa;
char *ptr;
char temp1[4096];
char temp2[4096];

if(debug_flag)
	{
	printf("rename_play_list(): arg line=%d\n", line);
	}

for(pa = play_listtab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		/* get new name from user */
		ptr = (char *) fl_show_input("New name for?", pa -> name);
		if(! ptr) return 0;		
		
		/* rename file on disk */
		a = rename(pa -> name, ptr);
		if(a == -1)
			{
			perror("rename_play_list() ");
			fl_show_alert("Could not rename", pa -> name, ptr, 0);

			return 0;
			}

		/* rename backup file if it exists*/
		sprintf(temp1, "%s~", pa -> name);
		sprintf(temp2, "%s~", ptr);
		a = rename(temp1, temp2);
		if(a == -1)
			{
			perror("rename_play_list() ");

			/* do nothing, perhaps no backup exists */
			}

		/* rename structure entry */
		if(pa -> name) free(pa -> name);
		pa -> name = strsave(ptr);
		if(! pa -> name) return 0;

		return 1;
		}

	}/* end for all entries */

/* not found */
return 0;
} /* end function rename_play_list */


