/* 
xmpl mp3 player frontend for mpg123
xmpl  is copyright (C) 2002 <Jan Panteltje>
email: panteltje@yahoo.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xmpl.h"

FD_xmpl *fdui;

FILE *filefd;
extern int debug_flag;


int save_setting(char *type, char *string)
{
char pathfilename[TEMP_SIZE];
FILE *setting_file;

/* combine with home directory */
sprintf(pathfilename, "%s/.xmpl/setup/%s", home_dir, type);
setting_file = fopen(pathfilename, "w");
if(! setting_file)
	{
	fl_show_alert("Cannot open file for write:", pathfilename, "", 0);
	if(debug_flag)
		{
		fprintf(stdout, "\nCannot open setting file %s for write\n",\
		pathfilename);
		}
	return 0;
	}

fprintf(setting_file, "%s", string);
fclose(setting_file);
return 1;	
}/* end function save_setting */


char *load_setting(char *type)
{
int a;
char temp[1024];
char pathfilename[TEMP_SIZE];
FILE *setting_file;

/* combine with home directory */
sprintf(pathfilename, "%s/.xmpl/setup/%s", home_dir, type);
setting_file = fopen(pathfilename, "r");
if(! setting_file)
	{
	fl_show_alert("Cannot open file for read:", pathfilename, "", 0);
	if(debug_flag)
		{
		fprintf(stdout, "\nCannot open setting file %s for read\n",\
		pathfilename);
		}
	return 0;
	}

a = readline(setting_file, temp);/* closes file if EOF */	
if(a != EOF) fclose(setting_file);
return(strsave(temp) );
}/* end function load_setting */


int save_fontsizes()
{
char temp[TEMP_SIZE];
sprintf(temp,\
"%d %d",\
main_form_fontsize,\
helpers_form_fontsize\
);

if(! save_setting("fontsizes", temp) ) return 0;

return 1;
}/* end function save_fontsizes */


int load_fontsizes()
{
char *data_read;
 
data_read = (char *) load_setting("fontsizes");
if(! data_read) return 0;
sscanf(data_read,
"%d %d",\
&main_form_fontsize,\
&helpers_form_fontsize\
);

free(data_read);

set_font_sizes(main_form_fontsize);

return 1;
}/* end function load_fontsizes */


int save_general_settings()
{
char *ptr;
char temp[TEMP_SIZE];

sprintf(temp, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d",\
tab_size,\
selected_title,\
repeat_mode,\
selected_volume,\
selected_audio_device,\
selected_play_list,\
visible_browser,\
auto_play_flag,\
alarm_mode,\
alarm_hour,\
alarm_minute,\
sleep_mode,\
allow_double_title_entries,\
power_down_flag);

save_setting("general_settings", temp);

save_setting("sound_dir", sound_dir);

ptr = (char *)fl_get_input(fdui -> main_form_search_input_field);
if(ptr)
	{
	save_setting("search_field", ptr);
	}

return 1;
}/* end function save_general_settings */


load_general_settings()
{
char temp[512];
char *ptr;

if(debug_flag)
	{
	fprintf(stdout, "load_general_settings(): arg none\n");
	}

ptr = load_setting("general_settings");
if(ptr)
	{
	sscanf(ptr, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d",\
	&tab_size,\
	&selected_title,\
	&repeat_mode,\
	&selected_volume,\
	&selected_audio_device,\
	&selected_play_list,\
	&visible_browser,\
	&auto_play_flag,\
	&alarm_mode,\
	&alarm_hour,\
	&alarm_minute,\
	&sleep_mode,\
	&allow_double_title_entries,\
	&power_down_flag);

	free(ptr);
	}

set_repeat_mode_button(repeat_mode);
set_repeat_mode_menu(repeat_mode);
set_alarm_menu();
set_double_entry_menu(allow_double_title_entries);

/* FORCE NO POWER DOWN! */
power_down_flag = 0;
set_power_down_menu(power_down_flag);

ptr = load_setting("sound_dir");
if(ptr)
	{
	free(sound_dir);
	sound_dir = ptr;
	}
else
	{
	sprintf(temp, "%s/", home_dir);
	sound_dir = strsave(temp);
	if(! sound_dir) return 0;
	}

set_mixer_volume(selected_volume);

set_audio_device(selected_audio_device);

set_auto_play_menu(auto_play_flag);

ptr = (char *) load_setting("search_field");
if(ptr)
	{
	fl_set_input(fdui -> main_form_search_input_field, ptr);
	}

return 1;
}/* end function load_general_settings */


int set_tabs(int tabsize)
{
int i;
char *space;

if(debug_flag)
	{
	fprintf(stdout, "set_tabs(): arg tabsize=%d\n", tabsize);
	}

/* argument check */
if(tabsize < 0) return 0;

/* set the global (used by expand_tabs ) */
tab_size = tabsize;

space = malloc(tabsize + 1);
if(! space) return 0;
for(i = 0; i < tabsize; i++)
	{
	space[i] = 'a';
	}
space[i] = 0;

fl_set_tabstop(space);
free(space);

return 1;
}/* end funcion set_tabs */


char *expand_tabs(char *text, int tabsize)
{
int c, i, j, k;
int tabs;
char *space;

if(debug_flag)
	{
	fprintf(stdout, "expand_tabs(): arg text=%s tabsize=%d\n",\
	text, tabsize);
	}

/* argument check */
if(! text) return 0;
if(tabsize < 0) return 0;

/* determine how many tabs, so we can calculate the required space */
i = 0;
tabs = 0;
while(1)
	{
	c = text[i];
	if(c == 0) break;
	if(c == 9) tabs++;
	i++;
	}/* end while all characters in text */

space = malloc(strlen(text) + (tabs * tabsize) - tabs + 1 + 1);
if(! space) return 0;

/* copy and expand */
i = 0;
j = 0;
while(1)
	{
	c = text[i];
	if(c == 9)
		{
		for(k = 0; k < tabsize; k++)
			{
			space[j] = ' ';
			j++;
			}
		i++;
		continue;
		}
	else
		{
		space[j] = c;
		}
	if(c == 0) break;
	i++;
	j++;
	}
return(space);
}/* end function expand_tabs */


int set_audio_device(int nr)
{
char text[80];
int color;

if(debug_flag)
	{
	printf("set_audio_device(): arg nr=%d\n", nr);
	}

switch(nr)
	{
	case AUDIO_DEVICE_1:
		strcpy(text, AUDIO_DEVICE1);
		color = FL_COL1;
		break;
	case AUDIO_DEVICE_2:
		strcpy(text, AUDIO_DEVICE2);
		color = FL_YELLOW;
		break;
	} /* end switch mode */

fl_set_object_color(fdui -> main_form_device_select_button, color, color);
fl_set_object_label(fdui -> main_form_device_select_button, text);

if(selected_audio_device_name) free(selected_audio_device_name);
selected_audio_device_name = strsave(text);
if(! selected_audio_device_name) return 0;

return 1;
} /* end function set_audio_device */


int set_repeat_mode_menu(int mode)
{
if(debug_flag)
	{
	printf("set_repeat_mode_menu(): arg mode=%d\n", mode);
	}

switch(mode)
	{
	case PLAY_REPEAT_NONE:
		fl_set_menu_item_mode(fdui -> main_form_title_list_menu,\
		SINGLE_PLAY, FL_PUP_CHECK);
		fl_set_menu_item_mode(fdui -> main_form_title_list_menu,\
		REPEAT_CURRENT, FL_PUP_BOX);
		fl_set_menu_item_mode(fdui -> main_form_title_list_menu,\
		REPEAT_SELECTION, FL_PUP_BOX);
		break;
	case PLAY_REPEAT_CURRENT:
		fl_set_menu_item_mode(fdui -> main_form_title_list_menu,\
		SINGLE_PLAY, FL_PUP_BOX);
		fl_set_menu_item_mode(fdui -> main_form_title_list_menu,\
		REPEAT_CURRENT, FL_PUP_CHECK);
		fl_set_menu_item_mode(fdui -> main_form_title_list_menu,\
		REPEAT_SELECTION, FL_PUP_BOX);
		break;
	case PLAY_REPEAT_SELECTION:
		fl_set_menu_item_mode(fdui -> main_form_title_list_menu,\
		SINGLE_PLAY, FL_PUP_BOX);
		fl_set_menu_item_mode(fdui -> main_form_title_list_menu,\
		REPEAT_CURRENT, FL_PUP_BOX);
		fl_set_menu_item_mode(fdui -> main_form_title_list_menu,\
		REPEAT_SELECTION, FL_PUP_CHECK);
		break;
	} /* end switch repeat_mode */

return 1;
} /* end function set_repeat_mode_menu */


int set_auto_play_menu(int flag)
{
if(debug_flag)
	{
	printf("set_auto_play_menu(): arg flag=%d\n", flag);
	}

switch(flag)
	{
	case 0:
		fl_set_menu_item_mode(fdui -> main_form_file_menu,\
		AUTO_PLAY_ON_START, FL_PUP_BOX);
		break;
	case 1:
		fl_set_menu_item_mode(fdui -> main_form_file_menu,\
		AUTO_PLAY_ON_START, FL_PUP_CHECK);
		break;
	} /* end switch flag */

return 1;
} /* end function set_auto_play_menu */


int set_double_entry_menu(int flag)
{
if(debug_flag)
	{
	printf("set_double_entry_menu(): arg flag=%d\n", flag);
	}

switch(flag)
	{
	case 0:
		fl_set_menu_item_mode(fdui -> main_form_title_list_menu,\
		ALLOW_DOUBLE_ENTRIES, FL_PUP_BOX);
		break;
	case 1:
		fl_set_menu_item_mode(fdui -> main_form_title_list_menu,\
		ALLOW_DOUBLE_ENTRIES, FL_PUP_CHECK);
		break;
	} /* end switch flag */

return 1;
} /* end function set_double_entry_menu */


int set_font_sizes()
{
int size;

if(debug_flag)
	{
	printf("set_font_sizes(): arg none\n\
	using main_form_fontsize=%d helpers_form_fontsize=%d\n",\
	main_form_fontsize, helpers_form_fontsize);
	}

/* helpers form */
size = integer_to_fontsize(helpers_form_fontsize);

/* text */
fl_set_object_lsize(fdui -> helpers_form_title, size);

/* intputs */
fl_set_object_lsize(fdui -> helpers_form_input, size);

/* buttons */
fl_set_object_lsize(fdui -> helpers_form_fontsize_up_button, size);
fl_set_object_lsize(fdui -> helpers_form_fontsize_down_button, size);
fl_set_object_lsize(fdui -> helpers_form_cancel_button, size);
fl_set_object_lsize(fdui -> helpers_form_accept_button, size);

/* main form */
size = integer_to_fontsize(main_form_fontsize);

/* browsers */
fl_set_browser_fontsize(fdui -> main_form_play_list_browser, size);
fl_set_browser_fontsize(fdui -> main_form_title_list_browser, size);

/* menus */
//fl_set_object_lsize(fdui -> main_form_window_menu, size);
fl_set_object_lsize(fdui -> main_form_file_menu, size);
fl_set_object_lsize(fdui -> main_form_play_list_menu, size);
fl_set_object_lsize(fdui -> main_form_title_list_menu, size);

/* text */
fl_set_object_lsize(fdui -> main_form_status_display, size);
fl_set_object_lsize(fdui -> main_form_title, size);

/* inputs */
fl_set_object_lsize(fdui -> main_form_search_input_field, size);

/* buttons */
fl_set_object_lsize(fdui -> main_form_play_list_button, size);
fl_set_object_lsize(fdui -> main_form_title_list_button, size);
fl_set_object_lsize(fdui -> main_form_device_select_button, size);
fl_set_object_lsize(fdui -> main_form_repeat_mode_button, size);
fl_set_object_lsize(fdui -> main_form_search_case_button, size);
fl_set_object_lsize(fdui -> main_form_search_button, size);
fl_set_object_lsize(fdui -> main_form_stop_button, size);
fl_set_object_lsize(fdui -> main_form_fontsize_up_button, size);
fl_set_object_lsize(fdui -> main_form_fontsize_down_button, size);
fl_set_object_lsize(fdui -> main_form_exit_switch, size);

/* sliders */
fl_set_object_lsize(fdui -> main_form_volume_slider, size);

return 1;
} /* end function set_font_sizes */


int set_mixer_volume(int volume)
{
FILE *pptr;
char temp[1024];
double da;

if(debug_flag)
	{
	printf("set_mixer_volume(): arg volume=%d\n", volume);
	}

da = volume / 1.0;
fl_set_slider_value(fdui -> main_form_volume_slider, da);

sprintf(temp, "cmix pcm %d\n", volume);
pptr = popen(temp, "r");
pclose(pptr);

return 1;
} /* end function set_mixer_volume */


int set_power_down_menu(int flag)
{
if(debug_flag)
	{
	printf("set_power_down_menu(): arg flag=%d\n", flag);
	}

switch(flag)
	{
	case 0:
		fl_set_menu_item_mode(fdui -> main_form_alarm_menu,\
		POWER_DOWN_PC_ON_SLEEP, FL_PUP_BOX);
		break;
	case 1:
		fl_set_menu_item_mode(fdui -> main_form_alarm_menu,\
		POWER_DOWN_PC_ON_SLEEP, FL_PUP_CHECK);
		break;
	} /* end switch flag */

return 1;
} /* end function set_power_down_menu */


