/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xmpl.h"

extern FD_xmpl *fdui;

int last_found_line;

struct title *lookup_title(char *name)
{
struct title *pa;

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(strcmp(pa -> name, name) == 0) return(pa);
	}

return 0;/*not found*/
}/* end function lookup_title */


struct title *install_title_at_end_of_list(char *name)
{
struct title *plast, *pnew;

if(debug_flag)
	{
	fprintf(stdout,\
	"install_title_at_end_off_list(): arg name=%s\n", name);
	}

pnew = lookup_title(name);
if(pnew) return(pnew);/* already there */

/* create new structure */
pnew = (struct title *) calloc(1, sizeof(*pnew) );
if(! pnew) return 0;
pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* get previous structure */
plast = titletab[1]; /* end list */

/* set new structure pointers */
pnew -> nxtentr = 0; /* new points top zero (is end) */
pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

/* set previuos structure pointers */
if( !titletab[0] ) titletab[0] = pnew; /* first element in list */
else plast -> nxtentr = pnew;

/* set array end pointer */
titletab[1] = pnew;

return(pnew);/* pointer to new structure */
}/* end function install_title_at_end_of_list */


int delete_title(int line)/* delete entry from double linked list */
{
struct title *pa, *pprev, *pdel, *pnext;
char name[80];

if(debug_flag)
	{
	fprintf(stdout, "delete_title(): arg line=%d\n", line);
	}

pa = titletab[0];
while(1)
	{
	/* if end list, return not found */
	if(! pa) return 0;

	if(line != pa -> browser_line)
		{
		/* point to next element in list */
		pa = pa -> nxtentr;

		/* loop for next element in list */
		continue;
		}

	/* we now know which struture to delete */
	pdel = pa;

	/* get previous and next structure */
	pnext = pa -> nxtentr;
	pprev = pa -> prventr;

	/* set pointers for previous structure */
	/* if first one, modify titletab[0] */
	if(pprev == 0) titletab[0] = pnext;
	else pprev -> nxtentr = pnext;

	/* set pointers for next structure */
	/* if last one, modify titletab[1] */
	if(pnext == 0) titletab[1] = pprev;
	else pnext -> prventr = pprev;
	
	/* delete structure */	
	free(pdel -> name);
	free(pdel); /* free structure */

	/* return OK deleted */
	return 1;
	}/* end for all structures */
}/* end function delete_title */


int delete_all_titles()/* delete all entries from table */
{
struct title *pa;

if(debug_flag)
	{
	fprintf(stdout, "delete_all_titles() arg none\n");
	}

while(1)
	{	
	pa = titletab[0];
	if(! pa) break;
	titletab[0] = pa -> nxtentr;

	free(pa -> name);
	free(pa);/* free structure */
	}/* end while all structures */

titletab[1] = 0;
return 1;
}/* end function delete_all_titles */


int load_titles()
{
int a;
FILE *fileptr;
char temp[READSIZE];
char pathfilename[TEMP_SIZE];
struct title *pa;
char name[TEMP_SIZE];
int browser_line; /* used to lookup name (= id) if clicked in display,
						not saved on disk, assigned each time list
						is displayed */
int end_title;
int status;
int line;
int spare;
unsigned long start_frame;
unsigned long length;
int repeats;
int volume;

if(debug_flag)
	{
	fprintf(stdout, "load_titles() arg none\n");
	}
	
delete_all_titles();

sprintf(pathfilename, "%s", selected_play_list_file_name);
fileptr = fopen(pathfilename, "r");
if(! fileptr)
	{
	fl_show_alert("could not open file", pathfilename, "for read", 1);

	if(debug_flag)
		{
		fprintf(stdout, "could not load file %s\n", pathfilename);
		}

	return 0;
	} 

/* for all lines in selected_play_list_filename */
line = 0;
while(1)
	{
	a = readline(fileptr, temp);
	if(a == EOF) return 1;/* empty file or last line */

	strcpy(name, temp);

	/* increment line counter for error reporting */
	line ++;

	a = readline(fileptr, temp);
	if(a == EOF) return 1;/* empty file or last line */

	start_frame = 0;	
	length = 0;
	repeats = 0;
	volume = 0;

	sscanf(temp,\
	"%d %d %lu %lu %d %d",\
	&browser_line,\
	&status,\
	&start_frame,\
	&length,\
	&repeats,\
	&volume);

	/* increment line counter for error reporting */
	line ++;

	/* create a structure entry */
	pa = install_title_at_end_of_list(name);
	if(! pa)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"load_titles: cannot install title %s\n", temp);
			}
		return 0;
		}

	pa -> status = status;
	pa -> browser_line = browser_line;
	pa -> start_frame = start_frame;
	pa -> length = length;
	pa -> repeats = repeats;
	pa -> volume = volume;
	}/* end while all lines in selected_play_list_filename */

return 1;
}/* end function load_titles */


int save_titles()
{
FILE *fileptr;
char pathfilename[TEMP_SIZE];
char pathfilename2[TEMP_SIZE];
struct title *pa;

if(debug_flag)
	{
	fprintf(stdout, "save_titles() arg none\n");
	}

sprintf(pathfilename, "%s.tmp", selected_play_list_file_name);
fileptr = fopen(pathfilename, "w");
if(! fileptr)
	{
	fl_show_alert("could not open file", pathfilename, "for write", 1);

	if(debug_flag)
		{
		fprintf(stdout,\
		"could not open file %s for write\n", pathfilename);
		}
	return 0;
	} 

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	fprintf(fileptr, "%s\n", pa -> name);

	fprintf(fileptr,\
	"%d %d %lu %lu %d %d\n",\
	pa -> browser_line,\
	pa -> status,\
	pa -> start_frame,\
	pa -> length,\
	pa -> repeats,\
	pa -> volume);
	}/* end while all elements in list */

fclose(fileptr);

if(debug_flag)
	{
	fprintf(stdout, "save_titles(): backing up %s\n",\
	selected_play_list_file_name);
	}

/* set some path file names */
sprintf(pathfilename, "%s", selected_play_list_file_name);

sprintf(pathfilename2, "%s~", selected_play_list_file_name);

/* unlink the old ~ */
unlink(pathfilename2);

/* rename to ~ */
if( rename(pathfilename, pathfilename2) == -1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "save_titles(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	}

/* rename .tmp */
sprintf(pathfilename2, "%s.tmp", selected_play_list_file_name);
if( rename(pathfilename2, pathfilename) == -1)
	{
	fl_show_alert("could not rename", pathfilename2, pathfilename, 1);

	if(debug_flag)
		{
		fprintf(stdout, "save_titles(); rename %s into %s failed\n",\
		pathfilename, pathfilename2);
		}
	return 0;
	}

return 1;/* ok */
}/* end function save_titles */


int show_titles(int position)
{
char temp[TEMP_SIZE];
struct title *pa;
char formatstr[20];
int browser_line;
int browser_topline;
char *strsave();
int first_new_title_line;
int find_first_new_title_flag;
int maxline;
int a;
int next_start_title;
struct title *pnext;
int hour, minute, second, title;
char start_str[80];
char length_str[80];
char repeats_str[80];
char volume_str[80];

if(debug_flag)
	{
	fprintf(stdout, "show_titles(): arg position=%d\n", position);
	}

/* remember vertical slider position */
browser_topline =\
	fl_get_browser_topline(fdui -> main_form_title_list_browser);

fl_freeze_form(fdui -> main_form);

fl_clear_browser(fdui -> main_form_title_list_browser);

first_new_title_line = -1;/* keep gcc -Wall from complaining */
find_first_new_title_flag = 1;
browser_line = 1;
for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	strcpy(formatstr, "@f");
	/* black is default color */
	strcat(formatstr, "@C0"); /* black */
//	if(!pa -> status) strcat(formatstr, "@C0"); /* black */
	if(pa -> status & NO_PLAY) strcat(formatstr, "@C18"); /* dark green */
//	if(pa -> status & TOO_LONG) strcat(formatstr, "@C1"); /* red */ 
//	if(pa -> status & TXT_HOLD) strcat(formatstr, "@C24"); /* blue */
//	if(pa -> status & XXXXXXX) strcat(formatstr, "@C5"); /* magenta */
	
//	if(highlight_titles(pa) ) strcat(formatstr, "@_");/* underscore */

	start_str[0] = 0;
	length_str[0] = 0;
	repeats_str[0] = 0;
	volume_str[0] = 0;
	if(pa -> start_frame) sprintf(start_str, "start=%lu", pa -> start_frame);
	if(pa -> length) sprintf(length_str, "length=%lu", pa -> length);
	if(pa -> repeats) sprintf(repeats_str, "repeats=%d", pa -> repeats);
	if(pa -> volume) sprintf(volume_str, "volume=%d", pa -> volume);

//	sprintf(temp,\
//	"%s %s start=%lu length=%lu repeats=%d",\
//	formatstr,\
//	pa -> name,\
//	pa -> start_frame,\
//	pa -> length,\
//	pa -> repeats);

	sprintf(temp,\
	"%s %s %s %s %s %s",\
	formatstr,\
	pa -> name,\
	start_str,\
	length_str,\
	repeats_str,\
	volume_str);
	
	fl_add_browser_line(fdui -> main_form_title_list_browser, temp);
	pa -> browser_line = browser_line;/* first one is 1 */

	browser_line++;
	}/* end for all elements in list */

/* re adjust browser for same position of top line (vertical slider) */	
if(position == SHOW_UNREAD_AT_TOP)
	{
	fl_set_browser_topline(fdui -> main_form_title_list_browser,\
	first_new_title_line);
	}
if(position == SHOW_BOTTOM)
	{
	/* make the status of the last title visible */
	maxline = fl_get_browser_maxline(fdui -> main_form_title_list_browser);
	fl_set_browser_topline(fdui -> main_form_title_list_browser, maxline);
	}
if(position == SHOW_TOP)
	{
	fl_set_browser_topline(fdui -> main_form_title_list_browser, 1);
	}
if(position == SHOW_SAME_POSITION)
	{
	fl_set_browser_topline(fdui -> main_form_title_list_browser,\
	browser_topline);
	}	
fl_unfreeze_form(fdui -> main_form);

return 1;
}/* end function show_titles */


char *line_to_filename(int line)
{
struct title *pa;

if(debug_flag)
	{
	fprintf(stdout, "line_to_filename(): arg line=%d\n", line);
	}

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)	
	{
	if(pa -> browser_line == line)
		{
		return pa -> name;
		}
	}

/* not found */
return 0;
}/* end function line_to_filename */


int title_to_line(char *text, int *line)
{
struct title *pa;
char name[50];

if(debug_flag)
	{
	fprintf(stdout, "title_to_line(): arg text=%s\n", text);
	}

pa = lookup_title(text);
if(! pa) return 0;

*line = pa -> browser_line;
return 1;
}/* end function title_to_line */


void replace(char *text, char *string, char *replace, char *result)
{
while(1)
	{
	if( (char *)strstr(text, string) == text)
		{
		*result = 0;
		strcat(result, replace);
		text += strlen(string);
		result += strlen(replace);
		}
	*result = *text;
	if(*text == 0)break;
	result++;
	text++;
	}
}/* end function replace */


int search_and_select_next_title()
{
int a;
struct title *pa;
char *keyword;
int case_sensitive_flag;
int found;

if(debug_flag)
	{
	fprintf(stdout, "search_and_select_next_title(): arg none\n");
	}

keyword = (char *)fl_get_input(fdui -> main_form_search_input_field);
if(! keyword) return 0;

case_sensitive_flag =\
fl_get_button(fdui -> main_form_search_case_button);

found = 0;
/* pa points to next entry */
for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* search from last position (set to zero if keyword changes */
	if(pa -> browser_line <= last_found_line) continue;
	
	if(search_in_for(pa -> name, keyword, case_sensitive_flag) )
		{
		found = 1;
		}

	if(found)
		{
		/*
		maintain visible indication that line is selected with single
		click
		*/
		fl_select_browser_line(fdui -> main_form_title_list_browser,\
		pa -> browser_line);

		/* show the entry */
		fl_set_browser_topline(fdui -> main_form_title_list_browser,\
		pa -> browser_line);

		/* remember where we are */
		last_found_line = pa -> browser_line;
		
		selected_title = atoi(pa -> name);

		/* return found */
		return 1;
		}/* end if found */
	}/* end for all title's */

if(last_found_line) fl_show_alert(keyword, "end of list", "", 0);
else fl_show_alert(keyword, "not found", "", 0);

last_found_line = 0;

/* return not found */
return 0;
}/* end function search_and_select_title */


int set_title_status(char *title, int status)
{
struct title *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"set_title_status(): arg title=%s status=%d\n", title, status);
	}

/* argument check */
if(! title) return 0;

pa = lookup_title(title);
if(! pa) return 0;

pa -> status = status;

return 1;
}/* end function set_title_status */
 

int get_title_status(char *title, int *status)
{
struct title *pa;

if(debug_flag)
	{
	fprintf(stdout,\
	"get_title_status(): arg title=%s\n", title);
	}

/* argument check */
if(! title) return 0;

pa = lookup_title(title);
if(! pa) return 0;

*status = pa -> status;

return 1;
}/* end function get_title_status */
 

int search_in_for(char *database, char *data, int case_sensitive)
{
/* This function returns 1 if all words in data are present in database */
char *ptra, *ptrb, *ptrc;
int match_failed_flag;
int last_string_flag;
char *strsave();
char *strstr_case_insensitve();

if(debug_flag)
	{
	fprintf(stdout,\
	"search_in_for(): arg database=%s data=%s case_sensitive=%d\n",\
	database, data, case_sensitive);
	}

/* argument check */
if(! database) return 0;
if(! data) return 0;
if( (case_sensitive != 0) && (case_sensitive != 1) ) return 0;

/*
Test if some data,
data may consist of one or more words separated by spaces
*/
if(! data[0]) return 0;/* no data */

/* test if data present in database */
/* for all entries in data */
ptra = data;
ptrb = strsave(data);/* create space */
match_failed_flag = 0;
last_string_flag = 0;
while(1)
	{
	ptrc = ptrb;/* point to start availeble space */
	while(1)/* string in data to ptrb */
		{
		*ptrc = *ptra;
		if(! *ptra) break;/* end data */
		if(*ptra == ' ')/* end of string in data */
			{
			*ptrc = 0;
			ptra++;/* scip the space */
			break;
			}
		ptra++;
		ptrc++;
		}/* end while all characters in string */
	/* test if string present in database */
	if(case_sensitive)
		{
		if(! strstr(database, ptrb) )
			{
			match_failed_flag = 1;
			break;
			}
		}
	else
		{
		if(! strstr_case_insensitive(database, ptrb) )
			{
			match_failed_flag = 1;
			break;
			}
		}/* end if not case sensitive */
	if(! *ptra) break;/* end filter */
	}/* end while all strings in filter */	
free(ptrb);/* release space */

if(match_failed_flag) return 0;
return 1;
} /* end function search_in_for */


char *strstr_case_insensitive(char *s, char *t)
{
int i, j, k;
for(i = 0; s[i] != '\0'; i++)
	{
	for(j = i, k = 0; t[k] != '\0' &&  toupper(s[j]) == toupper(t[k]);\
	j++, k++)
	;
	if(t[k] == '\0') return(s + i);
	}
return 0;
}/* end function strstr_case_insensitive */


struct title *install_title_in_position(char *name)
{
int i;
struct title *pa, *pprev, *plast, *plastmatch, *pnew, *pnext;
struct title *lookup_title();
char *test_string;
char *ptr1, *ptr2;

if(debug_flag)
	{
	fprintf(stdout,\
	"install_title_in_position():\nname=%s\n",\
	name);
	}
	
/* argument check */
if(! name) return 0;

plastmatch = 0;
for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	/* test if title with this name already exists */
	if(! allow_double_title_entries)
		{
		if(strcmp(name, pa -> name) == 0)
			{
			if(report_flag)
				{
				fl_show_alert(\
				"A title with name", name, "already exists, command cancelled", 0);
				}

			return 0;
			
			}
		}

	/* test if to far */
	if(strcmp(name, pa -> name) < 0) break;

	/* remember this position */
	plastmatch = pa;
	} /* end for all entries in list */

if(debug_flag)
	{
	fprintf(stdout,\
	"ouside for plastmatch=%lu\n", plastmatch);
	}

/* plastmatch now points to insert point, or zero if not found, */

/* create new structure */
pnew = (struct title *) calloc(1, sizeof(*pnew) );
if(! pnew) return 0;
pnew -> name = strsave(name);
if(! pnew -> name) return 0;

/* add to end list if no match */
if(! plastmatch)
	{
	/* get previous structure */
	plast = titletab[1]; /* end list */

	/* set new structure pointers */
	pnew -> nxtentr = 0; /* new points top zero (is end) */
	pnew -> prventr = plast; /* point to previous entry, or 0 if first entry */

	/* set previous structure pointers */
	if( !titletab[0] ) titletab[0] = pnew; /* first element in list */
	else plast -> nxtentr = pnew;

	/* set array end pointer */
	titletab[1] = pnew;

	/* return pointer to new structure */
	return(pnew);
	}/* end if ! plastmatch */

/* plastmatch now holds insert position, insert title */
/* insert the structure AFTER plastmatch */
		
/* get previous and next structure */
pnext = plastmatch -> nxtentr;
pprev = plastmatch;

/* set pointers for new struture */
pnew -> nxtentr = pnext;
pnew -> prventr = pprev;

/* set pointers for next structure */
if(pnext == 0) titletab[1] = pnew;
else pnext -> prventr = pnew;		

/* set pointer for previous structure */
if(pprev == 0) titletab[0] = pnew;
else pprev -> nxtentr = pnew;
		
/* mark it as a new entry */
pnew -> status = NEW_ENTRY;

/* return pointer to new structure */
return pnew;
}/* end function install_title_in_position */


int add_title_in_position(char *name)
{
char temp[512];
struct title *pa;
struct title *pnext;
int next_start_title;

if(debug_flag)
	{
	printf("add_title_in_position(): arg name=%s\n", name);
	}

/* sets status to new entry if found not to exist */
pa = install_title_in_position(name);
if(! pa) return 0;

pa -> start_frame = 0;
pa -> length = 0;
/* do not sort, user can move enrtries around */
//sort_titles();

return 1;
}/* end function add_title_in_position */


int sort_titles()
/*
sorts the double linked list with criterium set by  mode.
doing some sort of bubble sort.
*/
{
struct title *pa;
struct title *pb;
int swap_flag;

while(1)/* go through list again and again */
	{
	swap_flag = 0;
	for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
		{
		pb = pa -> prventr;
		if(pb)
			{
			if(strcmp(pa -> name, pb -> name) < 0)
				{
				/* indicate position was swapped */
				swap_flag = swap_title_position(pa , pb);
				}/* end if condition */
			}/* end if pb */
		}/* end for all entries */

	/* if no more swapping took place, ready, list is sorted */
	if(! swap_flag) break;
	}/* end while go through list again and again */

return 1;
}/* end function sort_titles */


int swap_title_position(struct title *ptop, struct title *pbottom)
{
struct title *punder;
struct title *pabove;

if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): swapping top=%lu bottom=%lu\n", ptop, pbottom);
	}

/* get one below the bottom */
punder = pbottom -> prventr;/* could be zero if first entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): punder=%lu\n", punder);
	}

/* get the one above the top */
pabove = ptop -> nxtentr;/* could be zero if last entry */
if(debug_flag)
	{
	fprintf(stdout,\
	"swap_position(): pabove=%lu\n", pabove);
	}

/* the next pointer in punder (or titletab[0]) must now point to ptop */
if(! punder)
	{
	titletab[0] = ptop; 
	}
else
	{
	punder -> nxtentr = ptop;
	}

/* the prev pointer in in ptop must now point to punder */
ptop -> prventr = punder;/* could be zero if first entry */

/* the next pointer in ptop must now point to pbottom */
ptop -> nxtentr = pbottom;

/* the next pointer in pbottom must now point to pabove */
pbottom -> nxtentr = pabove;

/* mark last one in titletabtab */
if(! pabove)
	{
	titletab[1] = pbottom;
	}
else
	{
	/* the prev pointer in pabove must now point to pbottom */
	pabove -> prventr = pbottom;
	}

/* the prev pointer in pbottom must now point to ptop */
pbottom -> prventr = ptop;

/* return swapped */
return 1;
}/* end function swap_title_position */


int swap_titles(int line)
/*
sorts the double linked list with criterium set by  mode.
doing some sort of bubble sort.
*/
{
struct title *pa;
struct title *pb;
int swap_flag;

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	pb = pa -> prventr;
	if(pb)
		{
		if(pa -> browser_line == line)
			{
			/* indicate position was swapped */
			swap_flag = swap_title_position(pa , pb);

			return 1;
			}/* end if condition */
		}/* end if pb */
	}/* end for all entries */

/* not found */
return 0;
}/* end funcion swap_titles */


int toggle_play_flag(int line)
{
struct title *pa;

if(debug_flag)
	{
	printf("toggle_play_flag(): arg line=%d\n", line);
	}

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		if(pa -> status & NO_PLAY) pa -> status &= ~NO_PLAY;
		else pa -> status |= NO_PLAY;

		save_titles();

		return 1;
		}
	}/* end for all entries */

/* not found */
return 0;
} /* end function toggle_play_flag */


int get_play_status(int line, int *status, int *volume)
{
struct title *pa;

if(debug_flag)
	{
	printf("get_play_status(): arg line=%d\n", line);
	}

*status = 0;
for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		*status = pa -> status;
		*volume = pa -> volume;

		return 1;
		}
	}/* end for all entries */

/* not found */
return 0;
} /* end function get_play_status */


int get_all_files(char *extention)
{
int a;
char temp[READSIZE];
char temp2[80];
FILE *pptr;
FILE *fptr;
char *ptr;
int error;

if(debug_flag)
	{
	printf("get_all_avis(): arg extention=%s\n", extention);
	}

/* sanity check */
error = 0;
if(! extention) error = 1;
if(strlen(extention) == 0) error = 1;
if(strlen(extention) > 70) error = 1;
if( strchr(extention, '.') ) error = 1;
if( strchr(extention, ' ') ) error = 1;
if( strchr(extention, '/') ) error = 1;
if( strchr(extention, '\\') ) error = 1;
if( strchr(extention, ',') ) error = 1;
if(error)
	{
	fl_show_alert(\
	"Incorrect extention specified",\
	"format is 'wav' without the quotes",\
	"command aborted",\
	0);

	return 0;
	}

/* make the list */
sprintf(temp, "locate %c.%s%c > %s/.xmpl/filelist.txt",\
'"', extention, '"', home_dir);
pptr = popen(temp, "r");
pclose(pptr);

/* read the created file list */
sprintf(temp, "%s/.xmpl/filelist.txt", home_dir);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fl_show_alert("Could not open file", temp, "No files found", 0);

	return 0;
	}

while(1)
	{
	/* closes file if EOF */
	a = readline(fptr, temp);
	if(a == EOF) break;

	/* filter for true lower- and upper case extentions */
	ptr = strrchr(temp, '.');

	/* not .anything */
	if(! ptr) continue; 
	
	/* ends not in .avi or .AVI */
	sprintf(temp2, ".%s", extention);
	if(strcasecmp(ptr, temp2) != 0) continue;

	/* add to structure, this will cancel double entries */
	add_title_in_position(temp);
	} /* end while all fonts in list */

return 1;
} /* end function int get_all_files */


int main_form_reset_last_found_line()
{
last_found_line = 0;

return 1;
}/* end function url_form_reset_last_found_line */


int get_start_frame(int line, unsigned long *frame)
{
struct title *pa;

if(debug_flag)
	{
	printf("get_start_frame(): arg line=%d\n", line);
	}

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		*frame = pa -> start_frame;

		return 1;
		}
	}/* end for all entries */

/* not found */
return 0;
} /* end function get_start_frame */


int set_start_frame(int line, unsigned long frame)
{
struct title *pa;

if(debug_flag)
	{
	printf("set_start_frame(): arg line=%d frame=%lu\n", line, frame);
	}

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		pa -> start_frame = frame;

		return 1;
		}
	}/* end for all entries */

/* not found */
return 0;
}/* end function set_start_frame */


int get_length(int line, unsigned long *length)
{
struct title *pa;

if(debug_flag)
	{
	printf("get_length(): arg line=%d\n", line);
	}

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		*length = pa -> length;

		return 1;
		}
	}/* end for all entries */

/* not found */
return 0;
}/* end function get_length */


int set_length(int line, unsigned long length) 
{
struct title *pa;

if(debug_flag)
	{
	printf("set_length(): arg line=%d length=%lu\n", line, length);
	}

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		pa -> length = length;

		return 1;
		}
	}/* end for all entries */

/* not found */
return 0;
}/* end function set_length */


int get_repeats(int line, int *repeats)
{
struct title *pa;

if(debug_flag)
	{
	printf("get_repeats(): arg line=%d\n", line);
	}

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		*repeats = pa -> repeats;

		return 1;
		}
	}/* end for all entries */

/* not found */
return 0;
}/* end function get_repeats */


int set_repeats(int line, int repeats) 
{
struct title *pa;

if(debug_flag)
	{
	printf("set_repeats(): arg line=%d repeats=%d\n", line, repeats);
	}

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		pa -> repeats = repeats;

		return 1;
		}
	}/* end for all entries */

/* not found */
return 0;
}/* end function set_repeats */


int title_to_start_length_repeats(\
char *title, unsigned long *start, unsigned long *length, int *repeats)
{
struct title *pa;
char name[50];

if(debug_flag)
	{
	printf("title_to_start_length_repeats(): arg title=%s\n", title);
	}

pa = lookup_title(title);
if(! pa) return 0;

*start = pa -> start_frame;
*length = pa -> length;
*repeats = pa -> repeats;

return 1;
}/* end function title_to_start_length_repeats */


int get_volume(int line, int *volume)
{
struct title *pa;

if(debug_flag)
	{
	printf("get_volume(): arg line=%d\n", line);
	}

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		*volume = pa -> volume;

		return 1;
		}
	}/* end for all entries */

/* not found */
return 0;
}/* end function get_volume */


int set_volume(int line, int volume) 
{
struct title *pa;

if(debug_flag)
	{
	printf("set_volume(): arg line=%d volume=%d\n", line, volume);
	}

for(pa = titletab[0]; pa != 0; pa = pa -> nxtentr)
	{
	if(pa -> browser_line == line)
		{
		pa -> volume = volume;

		return 1;
		}
	}/* end for all entries */

/* not found */
return 0;
}/* end function set_volume */


